"use strict";

module.exports = [
	/Can't resolve 'non-exported-css'/,
	/Can't resolve '\.\/directory'/,
	/Can't resolve 'condition-names-subpath\/non-valid\.css'/,
	/Can't resolve '\.\/no-extension-in-request'/
];
